@@ -3,6 +3,7 @@
  * Authors : 
  *  JP Rosevear <jpr@ximian.com>
  *  Rodrigo Moya <rodrigo@ximian.com>
+ *  Harish Krishnaswamy <kharish@novell.com>
  *
  * Copyright 2003, Novell, Inc.
  *
@@ -48,6 +49,16 @@
 #define O_BINARY 0
 #endif
 
+#ifdef G_OS_WIN32
+/* Undef the similar macro from pthread.h, it doesn't check if
+ * gmtime() returns NULL.
+ */
+#undef gmtime_r
+
+/* The gmtime() in Microsoft's C library is MT-safe */
+#define gmtime_r(tp,tmp) (gmtime(tp)?(*(tmp)=*gmtime(tp),(tmp)):0)
+#endif
+
 /* Private part of the CalBackendGroupwise structure */
 struct _ECalBackendGroupwisePrivate {
 	/* A mutex to control access to the private structure */
@@ -81,6 +92,7 @@ static ECalBackendSyncStatus
 e_cal_backend_groupwise_add_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzobj);
 static const char * get_gw_item_id (icalcomponent *icalcomp);
 static void get_retract_data (ECalComponent *comp, const char **retract_comment, gboolean *all_instances);
+static const char * get_element_type (icalcomponent_kind kind);
 
 #define PARENT_TYPE E_TYPE_CAL_BACKEND_SYNC
 static ECalBackendClass *parent_class = NULL;
@@ -124,6 +136,23 @@ e_cal_backend_groupwise_get_default_zone (ECalBackendGroupwise *cbgw) {
 
 static GMutex *mutex = NULL;
 
+static const char * 
+get_element_type (icalcomponent_kind kind) 
+{
+	
+	const char *type;
+
+	if (kind == ICAL_VEVENT_COMPONENT)
+		type = "Appointment";
+	else if (kind == ICAL_VTODO_COMPONENT)
+		type = "Task";
+	else
+		type = "Note";
+
+	return type;
+
+}
+
 /* Initialy populate the cache from the server */
 static EGwConnectionStatus
 populate_cache (ECalBackendGroupwise *cbgw)
@@ -132,93 +161,133 @@ populate_cache (ECalBackendGroupwise *cbgw)
 	EGwConnectionStatus status;
         ECalComponent *comp;
         GList *list = NULL, *l;
-	gboolean done = FALSE;
+	gboolean done = FALSE,  forward = FALSE;
 	int cursor = 0;
 	guint32	total, num = 0;
-	int percent = 0;
+	int percent = 0, i;
 	const char *position = E_GW_CURSOR_POSITION_END; 
 	icalcomponent_kind kind;
 	const char *type;
+	EGwFilter* filter[3];
+	char l_str[26]; 
+	char h_str[26];
+	icaltimetype temp;
+	struct tm tm;
+	time_t h_time, l_time;
 
 	priv = cbgw->priv;
 	kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbgw));
 	total = priv->total_count;
-	
+
 	if (!mutex) {
 		mutex = g_mutex_new ();
 	}
 
 	g_mutex_lock (mutex);
 
-	if (kind == ICAL_VEVENT_COMPONENT)
-		type = "Calendar";
-	else
-		type = "Task";
+	type = get_element_type (kind);	
 	
-	status = e_gw_connection_create_cursor (priv->cnc,
-			priv->container_id, 
-			"recipients message recipientStatus attachments default peek", NULL, &cursor);
-	if (status != E_GW_CONNECTION_STATUS_OK) {
-		e_cal_backend_groupwise_notify_error_code (cbgw, status);
-		g_mutex_unlock (mutex);
-                return status;
-        }
-	
-	while (!done) {
-		
-		status = e_gw_connection_read_cursor (priv->cnc, priv->container_id, cursor, FALSE, CURSOR_ITEM_LIMIT, position, &list);
+	/* Fetch the data with a bias to present, near past/future */
+	temp = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
+	i = g_ascii_strtod (g_getenv ("PRELOAD_WINDOW_DAYS")? g_getenv ("PRELOAD_WINDOW_DAYS"):"15", NULL);
+	temp.day -= i;
+	icaltime_normalize (temp);
+	l_time = icaltime_as_timet_with_zone (temp, icaltimezone_get_utc_timezone ());
+	gmtime_r (&l_time, &tm);
+	strftime (l_str, 26, "%Y-%m-%dT%H:%M:%SZ", &tm);
+	temp.day += (2*i);
+	icaltime_normalize (temp);
+	h_time = icaltime_as_timet_with_zone (temp, icaltimezone_get_utc_timezone ());
+	gmtime_r (&h_time, &tm);
+	strftime (h_str, 26, "%Y-%m-%dT%H:%M:%SZ", &tm);
+
+	filter[0] = e_gw_filter_new ();
+	e_gw_filter_add_filter_component (filter[0], E_GW_FILTER_OP_GREATERTHAN_OR_EQUAL, "startDate", l_str);
+	e_gw_filter_add_filter_component (filter[0], E_GW_FILTER_OP_LESSTHAN_OR_EQUAL, "startDate", h_str);
+	e_gw_filter_add_filter_component (filter[0], E_GW_FILTER_OP_EQUAL, "@type", type);
+	e_gw_filter_group_conditions (filter[0], E_GW_FILTER_OP_AND, 3);
+	filter[1] = e_gw_filter_new ();
+	e_gw_filter_add_filter_component (filter[1], E_GW_FILTER_OP_GREATERTHAN, "startDate", h_str);
+	e_gw_filter_add_filter_component (filter[1], E_GW_FILTER_OP_EQUAL, "@type", type);
+	e_gw_filter_group_conditions (filter[1], E_GW_FILTER_OP_AND, 2);
+	filter[2] = e_gw_filter_new ();
+	e_gw_filter_add_filter_component (filter[2], E_GW_FILTER_OP_LESSTHAN, "startDate", l_str);
+	e_gw_filter_add_filter_component (filter[2], E_GW_FILTER_OP_EQUAL, "@type", type);
+	e_gw_filter_group_conditions (filter[2], E_GW_FILTER_OP_AND, 2);
+
+	for (i = 0; i < 3; i++) {
+		status = e_gw_connection_create_cursor (priv->cnc,
+				priv->container_id, 
+				"recipients message recipientStatus attachments default peek", filter[i], &cursor);
 		if (status != E_GW_CONNECTION_STATUS_OK) {
 			e_cal_backend_groupwise_notify_error_code (cbgw, status);
 			g_mutex_unlock (mutex);
 			return status;
 		}
-		for (l = list; l != NULL; l = g_list_next(l)) {
-			EGwItem *item;
-			char *progress_string = NULL;
-			
-			item = E_GW_ITEM (l->data);
-			comp = e_gw_item_to_cal_component (item, cbgw);
-			g_object_unref (item);
-			
-			/* Show the progress information */
-			num++;
-			percent = ((float) num/total) * 100;
-		
-			/* FIXME The total obtained from the server is wrong. Sometimes the num can 
-			be greater than the total. The following makes sure that the percentage is not >= 100 */
- 
-			if (percent > 100)
-				percent = 99; 
+		done = FALSE;
+		if (i == 1) {
+			position = E_GW_CURSOR_POSITION_START;
+			forward = TRUE;
 
-			progress_string = g_strdup_printf (_("Loading %s items"), type);
-			e_cal_backend_notify_view_progress (E_CAL_BACKEND (cbgw), progress_string, percent);
+		} else {
+			position = E_GW_CURSOR_POSITION_END;
+			forward = FALSE;
+		}
 			
-			if (E_IS_CAL_COMPONENT (comp)) {
-				char *comp_str;
+		while (!done) {
+			
+			status = e_gw_connection_read_cursor (priv->cnc, priv->container_id, cursor, forward, CURSOR_ITEM_LIMIT, position, &list);
+			if (status != E_GW_CONNECTION_STATUS_OK) {
+				e_cal_backend_groupwise_notify_error_code (cbgw, status);
+				g_mutex_unlock (mutex);
+				return status;
+			}
+			for (l = list; l != NULL; l = g_list_next(l)) {
+				EGwItem *item;
+				char *progress_string = NULL;
 				
-				e_cal_component_commit_sequence (comp);
-				if (kind == icalcomponent_isa (e_cal_component_get_icalcomponent (comp))) {
+				item = E_GW_ITEM (l->data);
+				comp = e_gw_item_to_cal_component (item, cbgw);
+				g_object_unref (item);
+				
+				/* Show the progress information */
+				num++;
+				percent = ((float) num/total) * 100;
+			
+				/* FIXME The total obtained from the server is wrong. Sometimes the num can 
+				be greater than the total. The following makes sure that the percentage is not >= 100 */
+	 
+				if (percent > 100)
+					percent = 99; 
+
+				progress_string = g_strdup_printf (_("Loading %s items"), type);
+				e_cal_backend_notify_view_progress (E_CAL_BACKEND (cbgw), progress_string, percent);
+				
+				if (E_IS_CAL_COMPONENT (comp)) {
+					char *comp_str;
+					
+					e_cal_component_commit_sequence (comp);
 					comp_str = e_cal_component_get_as_string (comp);	
 					e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), (const char *) comp_str);
 					g_free (comp_str);
+					e_cal_backend_cache_put_component (priv->cache, comp);
+					g_object_unref (comp);
 				}
-				e_cal_backend_cache_put_component (priv->cache, comp);
-				g_object_unref (comp);
+				g_free (progress_string);
 			}
-			g_free (progress_string);
+			
+			if (!list  || g_list_length (list) == 0)
+				done = TRUE;
+			g_list_free (list);
+			list = NULL;
+			position = E_GW_CURSOR_POSITION_CURRENT;
 		}
-		
-		if (!list  || g_list_length (list) == 0)
-			done = TRUE;
-		g_list_free (list);
-		list = NULL;
-		position = E_GW_CURSOR_POSITION_CURRENT;
-        }
-	e_gw_connection_destroy_cursor (priv->cnc, priv->container_id, cursor);
+		e_gw_connection_destroy_cursor (priv->cnc, priv->container_id, cursor);
+		g_object_unref (filter[i]);
+	}
 	e_cal_backend_notify_view_done (E_CAL_BACKEND (cbgw), GNOME_Evolution_Calendar_Success);
 
 	g_mutex_unlock (mutex);
-
 	return E_GW_CONNECTION_STATUS_OK;
 }
 
@@ -241,7 +310,7 @@ get_deltas (gpointer handle)
 	GSList *cache_keys = NULL;
 	GPtrArray *uid_array = g_ptr_array_new ();
 	char *time_string = NULL;
-	char t_str [100]; 
+	char t_str [26]; 
 	const char *serv_time;
 	static GStaticMutex connecting = G_STATIC_MUTEX_INIT;
 	const char *time_interval_string;
@@ -255,7 +324,7 @@ get_deltas (gpointer handle)
 	icaltimetype temp;
 	gboolean done = FALSE;
 	int cursor = 0;
-	struct tm *tm;
+	struct tm tm;
 	time_t current_time;
 	gboolean needs_to_get = FALSE;
 
@@ -277,28 +346,30 @@ get_deltas (gpointer handle)
 
 	serv_time = e_cal_backend_cache_get_server_utc_time (cache);
 	if (serv_time) {
-		g_strlcpy (t_str, e_cal_backend_cache_get_server_utc_time (cache), 100);
+		g_strlcpy (t_str, e_cal_backend_cache_get_server_utc_time (cache), 26);
 		if (!*t_str || !strcmp (t_str, "")) {
 			/* FIXME: When time-stamp is crashed, getting changes from current time */
 			g_warning ("\n\a Could not get the correct time stamp. \n\a");
 			temp = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
 			current_time = icaltime_as_timet_with_zone (temp, icaltimezone_get_utc_timezone ());
-			tm = gmtime (&current_time);
-			strftime (t_str, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+			gmtime_r (&current_time, &tm);
+			strftime (t_str, 26, "%Y-%m-%dT%H:%M:%SZ", &tm);
 		}
 	} else {
 		/* FIXME: When time-stamp is crashed, getting changes from current time */
 		g_warning ("\n\a Could not get the correct time stamp. \n\a");
 		temp = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
 		current_time = icaltime_as_timet_with_zone (temp, icaltimezone_get_utc_timezone ());
-		tm = gmtime (&current_time);
-		strftime (t_str, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+		gmtime_r (&current_time, &tm);
+		strftime (t_str, 26, "%Y-%m-%dT%H:%M:%SZ", &tm);
 	}
 	time_string = g_strdup (t_str);
 
 	filter = e_gw_filter_new ();
 	/* Items modified after the time-stamp */
 	e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_GREATERTHAN, "modified", time_string);
+	e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_EQUAL, "@type", get_element_type (kind));
+	e_gw_filter_group_conditions (filter, E_GW_FILTER_OP_AND, 2);
 
 	status = e_gw_connection_get_items (cnc, cbgw->priv->container_id, "attachments recipients message recipientStatus default peek", filter, &item_list);
 	if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
@@ -351,12 +422,10 @@ get_deltas (gpointer handle)
 		e_cal_component_commit_sequence (modified_comp);
 		e_cal_component_commit_sequence (cache_comp);
 
-		if (kind == icalcomponent_isa (e_cal_component_get_icalcomponent (modified_comp))) {
-			cache_comp_str = e_cal_component_get_as_string (cache_comp);
-			e_cal_backend_notify_object_modified (E_CAL_BACKEND (cbgw), cache_comp_str, e_cal_component_get_as_string (modified_comp));
-			g_free (cache_comp_str);
-			cache_comp_str = NULL;
-		}
+		cache_comp_str = e_cal_component_get_as_string (cache_comp);
+		e_cal_backend_notify_object_modified (E_CAL_BACKEND (cbgw), cache_comp_str, e_cal_component_get_as_string (modified_comp));
+		g_free (cache_comp_str);
+		cache_comp_str = NULL;
 		e_cal_backend_cache_put_component (cache, modified_comp);
 
 		g_object_unref (item);
@@ -366,7 +435,7 @@ get_deltas (gpointer handle)
 
 	temp = icaltime_from_string (time_string);
 	current_time = icaltime_as_timet_with_zone (temp, icaltimezone_get_utc_timezone ());
-	tm = gmtime (&current_time);
+	gmtime_r (&current_time, &tm);
 
 	time_interval = (CACHE_REFRESH_INTERVAL / 60000);
 	time_interval_string = g_getenv ("GETQM_TIME_INTERVAL");
@@ -374,12 +443,12 @@ get_deltas (gpointer handle)
 		time_interval = g_ascii_strtod (time_interval_string, NULL);
 	} 
 	if (attempts) {
-		tm->tm_min += (time_interval * g_ascii_strtod (attempts, NULL));
+		tm.tm_min += (time_interval * g_ascii_strtod (attempts, NULL));
 		e_cal_backend_cache_put_key_value (cache, key, NULL);
 	} else {
-		tm->tm_min += time_interval;
+		tm.tm_min += time_interval;
 	}
-	strftime (t_str, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+	strftime (t_str, 26, "%Y-%m-%dT%H:%M:%SZ", &tm);
 	time_string = g_strdup (t_str);
 
 	e_cal_backend_cache_put_server_utc_time (cache, time_string);
@@ -396,7 +465,11 @@ get_deltas (gpointer handle)
 	 * checking for deleted items.*/
 	position = E_GW_CURSOR_POSITION_END;
 	cursor = 0;
-	status = e_gw_connection_create_cursor (cnc, cbgw->priv->container_id, "id iCalId recurrenceKey startDate", NULL, &cursor);
+	filter = e_gw_filter_new ();
+	e_gw_filter_add_filter_component (filter, E_GW_FILTER_OP_EQUAL, "@type", get_element_type (kind));
+
+	status = e_gw_connection_create_cursor (cnc, cbgw->priv->container_id, "id iCalId recurrenceKey startDate", filter, &cursor);
+	g_object_unref (filter);
 
 	if (status != E_GW_CONNECTION_STATUS_OK) {
 		if (status == E_GW_CONNECTION_STATUS_NO_RESPONSE) {
@@ -446,6 +519,7 @@ get_deltas (gpointer handle)
 		GCompareFunc func = NULL;
 		GSList *remove = NULL;
 		char *real_key = NULL;
+		const char *recur_key;
 
 		if (calid->recur_key && calid->ical_id) {
 			const char *rid = NULL;
@@ -459,12 +533,15 @@ get_deltas (gpointer handle)
 			real_key = g_strconcat (calid->recur_key, "@", rid, NULL);
 		}
 		
-		if (!calid->recur_key || real_key) 
+		if (!calid->recur_key || real_key) {
+			recur_key = real_key;
 			func = (GCompareFunc) strcmp;
-		else
+		} else {
+			recur_key = calid->recur_key;
 			func = (GCompareFunc) compare_prefix;
+		}
 
-		if (!(remove = g_slist_find_custom (cache_keys, calid->recur_key ? real_key :
+		if (!(remove = g_slist_find_custom (cache_keys, calid->recur_key ? recur_key :
 						calid->ical_id,  func))) {
 			g_ptr_array_add (uid_array, g_strdup (calid->item_id));
 			needs_to_get = TRUE;
@@ -552,7 +629,6 @@ get_deltas (gpointer handle)
 	}
 	
 	g_static_mutex_unlock (&connecting);
-
 	return TRUE;        
 }
 
@@ -620,7 +696,6 @@ cache_init (ECalBackendGroupwise *cbgw)
 	int time_interval;
 
 	kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbgw));
-	
 	time_interval = CACHE_REFRESH_INTERVAL;
 	time_interval_string = g_getenv ("GETQM_TIME_INTERVAL");
 	if (time_interval_string) {
@@ -660,9 +735,9 @@ cache_init (ECalBackendGroupwise *cbgw)
 			e_cal_backend_cache_put_server_utc_time (priv->cache, utc_str);
 
 			/*  Set up deltas only if it is a Calendar backend */
-			if (kind == ICAL_VEVENT_COMPONENT)
-				priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas_timeout, (gpointer) cbgw);
+			priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas_timeout, (gpointer) cbgw);
 			priv->mode = CAL_MODE_REMOTE;
+
 			return GNOME_Evolution_Calendar_Success;
 		}
 
@@ -687,8 +762,7 @@ cache_init (ECalBackendGroupwise *cbgw)
 		
 		/* get the deltas from the cache */
 		if (get_deltas (cbgw)) {
-			if (kind == ICAL_VEVENT_COMPONENT)
-				priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas_timeout, (gpointer) cbgw);
+			priv->timeout_id = g_timeout_add (time_interval, (GSourceFunc) get_deltas_timeout, (gpointer) cbgw);
 			priv->mode = CAL_MODE_REMOTE;
 			return GNOME_Evolution_Calendar_Success;
 		} else {
@@ -697,7 +771,6 @@ cache_init (ECalBackendGroupwise *cbgw)
 			return GNOME_Evolution_Calendar_PermissionDenied;	
 		}
 	}
-	
 }
 
 static ECalBackendSyncStatus
@@ -716,6 +789,7 @@ set_container_id_with_count (ECalBackendGroupwise *cbgw)
 	switch (kind) {
 	case ICAL_VEVENT_COMPONENT:
 	case ICAL_VTODO_COMPONENT:
+	case ICAL_VJOURNAL_COMPONENT:
 		e_source_set_name (e_cal_backend_get_source (E_CAL_BACKEND (cbgw)), _("Calendar"));
 		break;
 	default:
@@ -755,6 +829,7 @@ connect_to_server (ECalBackendGroupwise *cbgw)
 	char *real_uri;
 	ECalBackendGroupwisePrivate *priv;
 	ESource *source;
+	ECalSourceType source_type;
 	const char *use_ssl;
 	char *http_uri;
 	int permissions;
@@ -774,117 +849,134 @@ connect_to_server (ECalBackendGroupwise *cbgw)
 	if (!real_uri) {
 		e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Invalid server URI"));
 		return GNOME_Evolution_Calendar_NoSuchCal;
-	} else {
-		parent_user = (char *) e_source_get_property (source, "parent_id_name");
+	} 
+
+	kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbgw));
+
+	parent_user = (char *) e_source_get_property (source, "parent_id_name");
+	/* create connection to server */
+	if (parent_user) {
+		EGwConnection *cnc;
 		/* create connection to server */
-		if (parent_user) {
-			EGwConnection *cnc;
-			/* create connection to server */
-			cnc = e_gw_connection_new (real_uri, parent_user, priv->password);
-			if (!E_IS_GW_CONNECTION(cnc) && use_ssl && g_str_equal (use_ssl, "when-possible")) {
-				http_uri = g_strconcat ("http://", real_uri + 8, NULL);
-				cnc = e_gw_connection_new (http_uri, parent_user, priv->password);
-				g_free (http_uri);
-			}
-
-			if (!cnc) {
-				e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Authentication failed"));
-				return GNOME_Evolution_Calendar_AuthenticationFailed;
-			}
-				
-			priv->cnc = e_gw_connection_get_proxy_connection (cnc, parent_user, priv->password, priv->username, &permissions);
-
-			g_object_unref(cnc);
-		
-			if (!priv->cnc) {
-				e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Authentication failed"));
-				return GNOME_Evolution_Calendar_AuthenticationFailed;
-			}
-
-			kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbgw));
-
-			cbgw->priv->read_only = TRUE;
-		
-			if (kind == ICAL_VEVENT_COMPONENT && (permissions & E_GW_PROXY_APPOINTMENT_WRITE) )
-				cbgw->priv->read_only = FALSE;
-			else if (kind == ICAL_VTODO_COMPONENT && (permissions & E_GW_PROXY_TASK_WRITE))
-				cbgw->priv->read_only = FALSE;
-
-		} else {
-
-			priv->cnc = e_gw_connection_new (
-					real_uri,
-					priv->username,
-					priv->password);
-
-			if (!E_IS_GW_CONNECTION(priv->cnc) && use_ssl && g_str_equal (use_ssl, "when-possible")) {
-				http_uri = g_strconcat ("http://", real_uri + 8, NULL);
-				priv->cnc = e_gw_connection_new (http_uri, priv->username, priv->password);
-				g_free (http_uri);
-			}
-			cbgw->priv->read_only = FALSE;
+		cnc = e_gw_connection_new (real_uri, parent_user, priv->password);
+		if (!E_IS_GW_CONNECTION(cnc) && use_ssl && g_str_equal (use_ssl, "when-possible")) {
+			http_uri = g_strconcat ("http://", real_uri + 8, NULL);
+			cnc = e_gw_connection_new (http_uri, parent_user, priv->password);
+			g_free (http_uri);
 		}
-		g_free (real_uri);
-			
-		if (priv->cnc && priv->cache && priv->container_id) {
-			char *utc_str;
-			priv->mode = CAL_MODE_REMOTE;
-			if (priv->mode_changed && !priv->timeout_id && (e_cal_backend_get_kind (E_CAL_BACKEND (cbgw)) == ICAL_VEVENT_COMPONENT)) {
-				GThread *thread1;
-				priv->mode_changed = FALSE;
 
-				thread1 = g_thread_create ((GThreadFunc) get_deltas, cbgw, FALSE, &error);
-				if (!thread1) {
-					g_warning (G_STRLOC ": %s", error->message);
-					g_error_free (error);
-
-					e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create thread for getting deltas"));
-					return GNOME_Evolution_Calendar_OtherError;
-				}
-				priv->timeout_id = g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) get_deltas_timeout, (gpointer)cbgw);
-			}
-			utc_str = (char *) e_gw_connection_get_server_time (priv->cnc);
-			e_cal_backend_cache_put_server_utc_time (priv->cache, utc_str);
-
-			return GNOME_Evolution_Calendar_Success;
-		}
-		priv->mode_changed = FALSE;
-
-		if (E_IS_GW_CONNECTION (priv->cnc)) {
-			ECalBackendSyncStatus status;
-			/* get the ID for the container */
-			if (priv->container_id)
-				g_free (priv->container_id);
-			
-			if ((status = set_container_id_with_count (cbgw)) != GNOME_Evolution_Calendar_Success) {
-				return status;
-			}
-		
-			priv->cache = e_cal_backend_cache_new (e_cal_backend_get_uri (E_CAL_BACKEND (cbgw)));
-			if (!priv->cache) {
-				g_mutex_unlock (priv->mutex);
-				e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create cache file"));
-				return GNOME_Evolution_Calendar_OtherError;
-			}
-
-			e_cal_backend_cache_put_default_timezone (priv->cache, priv->default_zone);
-			e_cal_backend_groupwise_add_timezone (E_CAL_BACKEND_SYNC (cbgw), NULL, (const char *) icaltimezone_get_tzid (priv->default_zone));
-
-			/* spawn a new thread for opening the calendar */
-			thread = g_thread_create ((GThreadFunc) cache_init, cbgw, FALSE, &error);
-			if (!thread) {
-				g_warning (G_STRLOC ": %s", error->message);
-				g_error_free (error);
-
-				e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create thread for populating cache"));
-				return GNOME_Evolution_Calendar_OtherError;
-			}
-
-
-		} else {
+		if (!cnc) {
 			e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Authentication failed"));
 			return GNOME_Evolution_Calendar_AuthenticationFailed;
 		}
+			
+		priv->cnc = e_gw_connection_get_proxy_connection (cnc, parent_user, priv->password, priv->username, &permissions);
+
+		g_object_unref(cnc);
+	
+		if (!priv->cnc) {
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Authentication failed"));
+			return GNOME_Evolution_Calendar_AuthenticationFailed;
+		}
+
+
+		cbgw->priv->read_only = TRUE;
+	
+		if (kind == ICAL_VEVENT_COMPONENT && (permissions & E_GW_PROXY_APPOINTMENT_WRITE) )
+			cbgw->priv->read_only = FALSE;
+		else if (kind == ICAL_VTODO_COMPONENT && (permissions & E_GW_PROXY_TASK_WRITE))
+			cbgw->priv->read_only = FALSE;
+		else if (kind == ICAL_VJOURNAL_COMPONENT && (permissions & E_GW_PROXY_NOTES_WRITE))
+			cbgw->priv->read_only = FALSE;
+
+	} else {
+
+		priv->cnc = e_gw_connection_new (
+				real_uri,
+				priv->username,
+				priv->password);
+
+		if (!E_IS_GW_CONNECTION(priv->cnc) && use_ssl && g_str_equal (use_ssl, "when-possible")) {
+			http_uri = g_strconcat ("http://", real_uri + 8, NULL);
+			priv->cnc = e_gw_connection_new (http_uri, priv->username, priv->password);
+			g_free (http_uri);
+		}
+		cbgw->priv->read_only = FALSE;
+	}
+	g_free (real_uri);
+		
+	if (priv->cnc && priv->cache && priv->container_id) {
+		char *utc_str;
+		priv->mode = CAL_MODE_REMOTE;
+		if (priv->mode_changed && !priv->timeout_id ) {
+			GThread *thread1;
+			priv->mode_changed = FALSE;
+
+			thread1 = g_thread_create ((GThreadFunc) get_deltas, cbgw, FALSE, &error);
+			if (!thread1) {
+				g_warning (G_STRLOC ": %s", error->message);
+				g_error_free (error);
+
+				e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create thread for getting deltas"));
+				return GNOME_Evolution_Calendar_OtherError;
+			}
+			priv->timeout_id = g_timeout_add (CACHE_REFRESH_INTERVAL, (GSourceFunc) get_deltas_timeout, (gpointer)cbgw);
+		}
+		utc_str = (char *) e_gw_connection_get_server_time (priv->cnc);
+		e_cal_backend_cache_put_server_utc_time (priv->cache, utc_str);
+
+		return GNOME_Evolution_Calendar_Success;
+	}
+	priv->mode_changed = FALSE;
+
+	switch (kind) {
+	case ICAL_VEVENT_COMPONENT:
+		source_type = E_CAL_SOURCE_TYPE_EVENT;
+		break;
+	case ICAL_VTODO_COMPONENT:
+		source_type = E_CAL_SOURCE_TYPE_TODO;
+		break;
+	case ICAL_VJOURNAL_COMPONENT:
+		source_type = E_CAL_SOURCE_TYPE_JOURNAL;
+		break;
+	default:
+		source_type = E_CAL_SOURCE_TYPE_EVENT;
+
+	}
+
+	if (E_IS_GW_CONNECTION (priv->cnc)) {
+		ECalBackendSyncStatus status;
+		/* get the ID for the container */
+		if (priv->container_id)
+			g_free (priv->container_id);
+		
+		if ((status = set_container_id_with_count (cbgw)) != GNOME_Evolution_Calendar_Success) {
+			return status;
+		}
+	
+		priv->cache = e_cal_backend_cache_new (e_cal_backend_get_uri (E_CAL_BACKEND (cbgw)), source_type);
+		if (!priv->cache) {
+			g_mutex_unlock (priv->mutex);
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create cache file"));
+			return GNOME_Evolution_Calendar_OtherError;
+		}
+
+		e_cal_backend_cache_put_default_timezone (priv->cache, priv->default_zone);
+
+		/* spawn a new thread for opening the calendar */
+		thread = g_thread_create ((GThreadFunc) cache_init, cbgw, FALSE, &error);
+		if (!thread) {
+			g_warning (G_STRLOC ": %s", error->message);
+			g_error_free (error);
+
+			e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create thread for populating cache"));
+			return GNOME_Evolution_Calendar_OtherError;
+		}
+
+
+	} else {
+		e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Authentication failed"));
+		return GNOME_Evolution_Calendar_AuthenticationFailed;
 	}
 
 	if (!e_gw_connection_get_version (priv->cnc)) 
@@ -966,6 +1058,11 @@ e_cal_backend_groupwise_finalize (GObject *object)
 		priv->timeout_id = 0;
 	}
 
+	if (priv->default_zone) {
+		icaltimezone_free (priv->default_zone, 1);
+		priv->default_zone = NULL;
+	}
+
 	g_free (priv);
 	cbgw->priv = NULL;
 
@@ -1082,6 +1179,8 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 	ECalBackendGroupwise *cbgw;
 	ECalBackendGroupwisePrivate *priv;
 	ECalBackendSyncStatus status;
+	ECalSourceType source_type;
+	char *source;
 	char *filename;
 	char *mangled_uri;
 	int i;
@@ -1093,6 +1192,23 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 
 	cbgw->priv->read_only = FALSE;
 
+	switch (e_cal_backend_get_kind (E_CAL_BACKEND (backend))) {
+	case ICAL_VEVENT_COMPONENT:
+		source_type = E_CAL_SOURCE_TYPE_EVENT;
+		source = "calendar";
+		break;
+	case ICAL_VTODO_COMPONENT:
+		source_type = E_CAL_SOURCE_TYPE_TODO;
+		source = "tasks";
+		break;
+	case ICAL_VJOURNAL_COMPONENT:
+		source_type = E_CAL_SOURCE_TYPE_JOURNAL;
+		source = "journal";
+		break;
+	default:
+		source_type = E_CAL_SOURCE_TYPE_EVENT;
+	}
+
 	if (priv->mode == CAL_MODE_LOCAL) {
 		ESource *source;
 		const char *display_contents = NULL;
@@ -1107,7 +1223,7 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 		}
 
 		if (!priv->cache) {
-			priv->cache = e_cal_backend_cache_new (e_cal_backend_get_uri (E_CAL_BACKEND (cbgw)));
+			priv->cache = e_cal_backend_cache_new (e_cal_backend_get_uri (E_CAL_BACKEND (cbgw)), source_type);
 			if (!priv->cache) {
 				g_mutex_unlock (priv->mutex);
 				e_cal_backend_notify_error (E_CAL_BACKEND (cbgw), _("Could not create cache file"));
@@ -1117,7 +1233,6 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 		}
 		
 		e_cal_backend_cache_put_default_timezone (priv->cache, priv->default_zone);
-		e_cal_backend_groupwise_add_timezone (backend, cal, (const char *) icaltimezone_get_tzid (priv->default_zone));
 
 		g_mutex_unlock (priv->mutex);	
 		return GNOME_Evolution_Calendar_Success;
@@ -1138,7 +1253,7 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 	}
 
 	filename = g_build_filename (g_get_home_dir (),
-				     ".evolution/cache/calendar",
+				     ".evolution/cache/", source,
 				     mangled_uri,
 				     NULL);
 	g_free (mangled_uri);
@@ -1372,16 +1487,32 @@ e_cal_backend_groupwise_add_timezone (ECalBackendSync *backend, EDataCal *cal, c
 }
 
 static ECalBackendSyncStatus
-e_cal_backend_groupwise_set_default_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzid)
+e_cal_backend_groupwise_set_default_zone (ECalBackendSync *backend, EDataCal *cal, const char *tzobj)
 {
+	icalcomponent *tz_comp;
 	ECalBackendGroupwise *cbgw;
-        ECalBackendGroupwisePrivate *priv;
+	ECalBackendGroupwisePrivate *priv;
+	icaltimezone *zone;
+
+	cbgw = (ECalBackendGroupwise *) backend;
+
+	g_return_val_if_fail (E_IS_CAL_BACKEND_GROUPWISE (cbgw), GNOME_Evolution_Calendar_OtherError);
+	g_return_val_if_fail (tzobj != NULL, GNOME_Evolution_Calendar_OtherError);
+
+	priv = cbgw->priv;
+
+	tz_comp = icalparser_parse_string (tzobj);
+	if (!tz_comp)
+		return GNOME_Evolution_Calendar_InvalidObject;
+
+	zone = icaltimezone_new ();
+	icaltimezone_set_component (zone, tz_comp);
+
+	if (priv->default_zone)
+		icaltimezone_free (priv->default_zone, 1);
 
-	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
-        priv = cbgw->priv;
-	
 	/* Set the default timezone to it. */
-	priv->default_zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
+	priv->default_zone = zone;
 
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -1528,6 +1659,7 @@ e_cal_backend_groupwise_compute_changes_foreach_key (const char *key, const char
 		be_data->deletes = g_list_prepend (be_data->deletes, e_cal_component_get_as_string (comp));
 
 		e_xmlhash_remove (be_data->ehash, key);
+		g_object_unref (comp);
  	}
 }
 
@@ -1622,9 +1754,10 @@ e_cal_backend_groupwise_discard_alarm (ECalBackendSync *backend, EDataCal *cal,
 
 static icaltimezone *
 e_cal_backend_groupwise_internal_get_default_timezone (ECalBackend *backend)
-{
-	/* Groupwise server maintains data in UTC  */
-	return icaltimezone_get_utc_timezone ();
+{	
+	ECalBackendGroupwise *cbgw = E_CAL_BACKEND_GROUPWISE (backend);
+	
+	return cbgw->priv->default_zone;
 }
 
 static icaltimezone *
@@ -1704,7 +1837,7 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomp);
-	
+
 	/* check if the object exists */
 	switch (priv->mode) {
 	case CAL_MODE_ANY :
@@ -1725,8 +1858,11 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 		}
 	
 		/* If delay deliver has been set, server will not send the uid */
-		if (!uid_list)
+		if (!uid_list || ((e_cal_component_get_vtype (comp) == E_CAL_COMPONENT_JOURNAL) && e_cal_component_has_organizer (comp))) {
+			*calobj = NULL;
+			g_object_unref (comp);
 			return GNOME_Evolution_Calendar_Success;
+		}
 		
 		if (g_slist_length (uid_list) == 1) {
 			server_uid = (char *) uid_list->data;
@@ -2608,7 +2744,7 @@ e_cal_backend_groupwise_class_init (ECalBackendGroupwiseClass *class)
 	sync_class->get_attachment_list_sync = e_cal_backend_groupwise_get_attachment_list;
 	sync_class->get_timezone_sync = e_cal_backend_groupwise_get_timezone;
 	sync_class->add_timezone_sync = e_cal_backend_groupwise_add_timezone;
-	sync_class->set_default_timezone_sync = e_cal_backend_groupwise_set_default_timezone;
+	sync_class->set_default_zone_sync = e_cal_backend_groupwise_set_default_zone;
 	sync_class->get_freebusy_sync = e_cal_backend_groupwise_get_free_busy;
 	sync_class->get_changes_sync = e_cal_backend_groupwise_get_changes;
 
