@@ -6,15 +6,15 @@
  * Authors: David Trowbridge <trowbrds@cs.colorado.edu>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
@@ -25,13 +25,39 @@
 
 #include <string.h>
 #include <stdlib.h>
-#include <glib/gi18n-lib.h>
+
 #include <libxml/parser.h>
 #include <libxml/tree.h>
+
+#include <glib/gi18n-lib.h>
+
+#include <gconf/gconf-client.h>
+
+#include "libedataserver/e-xml-utils.h"
+
 #include "e-weather-source-ccf.h"
 
 #define DATA_SIZE 5000
 
+#ifdef G_OS_WIN32
+
+#include <libedataserver/e-data-server-util.h>
+
+/* The localtime_r() in <pthread.h> doesn't guard against localtime()
+ * returning NULL
+ */
+#undef localtime_r
+
+/* The localtime() in Microsoft's C library is MT-safe */
+#define localtime_r(tp,tmp) (localtime(tp)?(*(tmp)=*localtime(tp),(tmp)):0)
+
+/* strtok() is also MT-safe (but not stateless, still uses only one
+ * buffer pointer per thread, but for the use of strtok_r() here
+ * that's enough).
+ */
+#define strtok_r(s,sep,lasts) (*(lasts)=strtok((s),(sep)))
+#endif
+
 static gchar *
 parse_for_url (char *code, char *name, xmlNode *parent)
 {
@@ -72,10 +98,20 @@ find_station_url (gchar *station, EWeatherSourceCCF *source)
 	xmlNode *root;
 	gchar **sstation;
 	gchar *url;
+	gchar *filename;
 
 	sstation = g_strsplit (station, "/", 2);
 
-	doc = xmlParseFile (WEATHER_DATADIR "/Locations.xml");
+#ifndef G_OS_WIN32
+	filename = g_strdup (WEATHER_DATADIR "/Locations.xml");
+#else
+	filename = e_util_replace_prefix (E_DATA_SERVER_PREFIX,
+					  e_util_get_prefix (),
+					  WEATHER_DATADIR "/Locations.xml");
+#endif
+
+	doc = e_xml_parse_file (filename);
+
 	g_assert (doc != NULL);
 
 	root = xmlDocGetRootElement (doc);
@@ -106,15 +142,13 @@ static GSList*
 tokenize (char *buffer)
 {
 	char *token;
-	char *tokbuf = g_strdup (buffer);
-	char *buffer2 = tokbuf;
+	char *tokbuf;
 	GSList *ret;
 
 	token = strtok_r (buffer, " \n", &tokbuf);
 	ret = g_slist_append (NULL, g_strdup (token));
 	while ((token = strtok_r (NULL, " \n/", &tokbuf)))
 		ret = g_slist_append (ret, g_strdup (token));
-	g_free (buffer2);
 	return ret;
 }
 
@@ -400,8 +434,49 @@ e_weather_source_ccf_parse (EWeatherSource *source, EWeatherSourceFinished done,
 
 	ccfsource->done = done;
 
-	if (!ccfsource->soup_session)
+	if (!ccfsource->soup_session) {
+		GConfClient *conf_client;
 		ccfsource->soup_session = soup_session_async_new ();
+
+		/* set the HTTP proxy, if configuration is set to do so */
+		conf_client = gconf_client_get_default ();
+		if (gconf_client_get_bool (conf_client, "/system/http_proxy/use_http_proxy", NULL)) {
+			char *server, *proxy_uri;
+			int port;
+
+			server = gconf_client_get_string (conf_client, "/system/http_proxy/host", NULL);
+			port = gconf_client_get_int (conf_client, "/system/http_proxy/port", NULL);
+
+			if (server && server[0]) {
+				SoupUri *suri;
+				if (gconf_client_get_bool (conf_client, "/system/http_proxy/use_authentication", NULL)) {
+					char *user, *password;
+
+					user = gconf_client_get_string (conf_client,
+									"/system/http_proxy/authentication_user",
+									NULL);
+					password = gconf_client_get_string (conf_client,
+									    "/system/http_proxy/authentication_password",
+									    NULL);
+
+					proxy_uri = g_strdup_printf("http://%s:%s@%s:%d", user, password, server, port);
+
+					g_free (user);
+					g_free (password);
+				} else
+					proxy_uri = g_strdup_printf ("http://%s:%d", server, port);
+
+				suri = soup_uri_new (proxy_uri);
+				g_object_set (G_OBJECT (ccfsource->soup_session), SOUP_SESSION_PROXY_URI, suri, NULL);
+
+				soup_uri_free (suri);
+				g_free (server);
+				g_free (proxy_uri);
+			}
+		}
+		g_object_unref (conf_client);
+	}
+
 	soup_message = soup_message_new (SOUP_METHOD_GET, ccfsource->url);
 	soup_message_set_flags (soup_message, SOUP_MESSAGE_NO_REDIRECT);
 	soup_session_queue_message (ccfsource->soup_session, soup_message, (SoupMessageCallbackFn) retrieval_done, source);
