@@ -165,13 +165,13 @@ writeln (CamelMimeFilter *filter, const char *in, const char *inend, char *outpt
 {
 	CamelMimeFilterToHTML *html = (CamelMimeFilterToHTML *) filter;
 	const char *inptr = in;
-
+	
 	while (inptr < inend) {
 		guint32 u;
-
+		
 		outptr = check_size (filter, outptr, outend, 16);
-
-		u = camel_utf8_getc_limit ((const unsigned char **) &inptr, inend);
+		
+		u = camel_utf8_getc_limit ((const unsigned char **) &inptr, (const unsigned char *) inend);
 		switch (u) {
 		case 0xffff:
 			g_warning("Truncated utf8 buffer");
