@@ -253,7 +253,7 @@ e_data_server_module_add_type (GType type)
 {
 	GObject *object;
 
-	g_printerr ("adding type `%s'\n", g_type_name (type));
+	g_message ("adding type `%s'", g_type_name (type));
 
 	object = g_object_new (type, NULL);
 	g_object_weak_ref (object, 
@@ -270,5 +270,5 @@ e_data_server_module_add_type (GType type)
 void
 e_data_server_module_remove_unused (void)
 {
-	g_list_foreach (module_objects, (GFunc)g_object_unref, NULL);
+	g_list_foreach (module_objects, g_object_unref, NULL);
 }
