@@ -58,12 +58,19 @@ typedef struct {
 	ENameSelectorDialog *dlg_ptr;
 } SelData;
 
+typedef struct _ENameSelectorDialogPrivate	ENameSelectorDialogPrivate;
+struct _ENameSelectorDialogPrivate
+{
+	guint destination_index;
+};
+
 static ESource *find_first_source             (ESourceList *source_list);
 static void     search_changed                (ENameSelectorDialog *name_selector_dialog);
 static void     source_selected               (ENameSelectorDialog *name_selector_dialog, ESource *source);
 static void     transfer_button_clicked       (ENameSelectorDialog *name_selector_dialog, GtkButton *transfer_button);
 static void     contact_selection_changed     (ENameSelectorDialog *name_selector_dialog);
 static void     setup_name_selector_model     (ENameSelectorDialog *name_selector_dialog);
+static void     shutdown_name_selector_model  (ENameSelectorDialog *name_selector_dialog);
 static void     contact_activated             (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path);
 static void     destination_activated         (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path,
 					       GtkTreeViewColumn *column, GtkTreeView *tree_view);
@@ -83,6 +90,10 @@ static void     destination_column_formatter  (GtkTreeViewColumn *column, GtkCel
 
 G_DEFINE_TYPE (ENameSelectorDialog, e_name_selector_dialog, GTK_TYPE_DIALOG);
 
+#define E_NAME_SELECTOR_DIALOG_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE ((obj), E_TYPE_NAME_SELECTOR_DIALOG, ENameSelectorDialogPrivate))
+
+
 static void
 e_name_selector_dialog_get_property (GObject *object, guint prop_id,
 				     GValue *value, GParamSpec *pspec)
@@ -145,6 +156,9 @@ e_name_selector_dialog_init (ENameSelectorDialog *name_selector_dialog)
 	ESourceList       *source_list;
 	char              *gladefile;
 
+	ENameSelectorDialogPrivate *priv = E_NAME_SELECTOR_DIALOG_GET_PRIVATE (name_selector_dialog);
+	priv->destination_index = 0;
+	
 	/* Get Glade GUI */
 	gladefile = g_build_filename (E_DATA_SERVER_UI_GLADEDIR,
 				      "e-name-selector-dialog.glade",
@@ -289,12 +303,9 @@ e_name_selector_dialog_dispose (GObject *object)
 {
 	ENameSelectorDialog *name_selector_dialog = E_NAME_SELECTOR_DIALOG (object);
 
-	g_signal_handlers_disconnect_matched (name_selector_dialog->name_selector_model,
-					      G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL,
-					      name_selector_dialog);
-
 	remove_books (name_selector_dialog);
-	
+	shutdown_name_selector_model (name_selector_dialog);
+
 	if (G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->dispose)
 		G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->dispose (object);
 }
@@ -303,7 +314,11 @@ e_name_selector_dialog_dispose (GObject *object)
 static void
 e_name_selector_dialog_finalize (GObject *object)
 {
-	/* TODO: Free stuff */
+	ENameSelectorDialog *name_selector_dialog = E_NAME_SELECTOR_DIALOG (object);
+
+	g_array_free (name_selector_dialog->sections, TRUE);
+	g_object_unref (name_selector_dialog->source_list);
+	g_object_unref (name_selector_dialog->button_size_group);
 	g_list_free (category_list);
 
 	if (G_OBJECT_CLASS (e_name_selector_dialog_parent_class)->finalize)
@@ -324,6 +339,8 @@ e_name_selector_dialog_class_init (ENameSelectorDialogClass *name_selector_dialo
 
 	/* Install signals */
 
+	g_type_class_add_private (object_class, sizeof(ENameSelectorDialogPrivate));
+
 }
 
 /**
@@ -417,6 +434,9 @@ remove_books (ENameSelectorDialog *name_selector_dialog)
 	GList         *books;
 	GList         *l;
 
+	if (!name_selector_dialog->name_selector_model)
+		return;
+
 	contact_store = e_name_selector_model_peek_contact_store (name_selector_dialog->name_selector_model);
 
 	/* Remove books (should be just one) being viewed */
@@ -522,6 +542,8 @@ add_section (ENameSelectorDialog *name_selector_dialog,
 	section.remove_button  = GTK_BUTTON (gtk_button_new());
 	section.destination_view = GTK_TREE_VIEW (gtk_tree_view_new ());
 
+	gtk_label_set_mnemonic_widget (GTK_LABEL (section.label), section.destination_view);
+
 	if (pango_parse_markup (pretty_name, -1, '_', NULL,
 				&text, NULL, NULL))  {
 		atk_object_set_name (gtk_widget_get_accessible (
@@ -759,10 +781,9 @@ search_changed (ENameSelectorDialog *name_selector_dialog)
 	category_option_menu = glade_xml_get_widget(name_selector_dialog->gui, "optionmenu-category");
 	category_id = gtk_option_menu_get_history (GTK_OPTION_MENU(category_option_menu));
 	category = g_list_nth_data (category_list, category_id);
-	category_escaped = escape_sexp_string (category);
-
 	if (!category)
 		return;
+	category_escaped = escape_sexp_string (category);
 
 	text = gtk_entry_get_text (name_selector_dialog->search_entry);
 	text_escaped = escape_sexp_string (text);
@@ -821,6 +842,8 @@ contact_activated (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path)
 	Section           *section;
 	gint               email_n;
 
+	ENameSelectorDialogPrivate *priv;
+
 	/* When a contact is activated, we transfer it to the first destination on our list */
 
 	contact_store = e_name_selector_model_peek_contact_store (name_selector_dialog->name_selector_model);
@@ -844,7 +867,9 @@ contact_activated (ENameSelectorDialog *name_selector_dialog, GtkTreePath *path)
 	}
 
 
-	section = &g_array_index (name_selector_dialog->sections, Section, 0);
+	priv = E_NAME_SELECTOR_DIALOG_GET_PRIVATE (name_selector_dialog);
+
+	section = &g_array_index (name_selector_dialog->sections, Section, priv->destination_index);
 	if (!e_name_selector_model_peek_section (name_selector_dialog->name_selector_model,
 						 section->name, NULL, &destination_store)) {
 		g_warning ("ENameSelectorDialog has a section unknown to the model!");
@@ -1038,14 +1063,6 @@ setup_name_selector_model (ENameSelectorDialog *name_selector_dialog)
 	ETreeModelGenerator *contact_filter;
 	GList               *new_sections;
 	GList               *l;
-	gint                 i;
-
-	/* Rid UI of previous destination sections */
-
-	for (i = 0; i < name_selector_dialog->sections->len; i++)
-		free_section (name_selector_dialog, i);
-
-	g_array_set_size (name_selector_dialog->sections, 0);
 
 	/* Create new destination sections in UI */
 
@@ -1081,9 +1098,6 @@ setup_name_selector_model (ENameSelectorDialog *name_selector_dialog)
 
 	/* Create sorting model on top of filter, assign it to view */
 
-	if (name_selector_dialog->contact_sort)
-		g_object_unref (name_selector_dialog->contact_sort);
-
 	name_selector_dialog->contact_sort = GTK_TREE_MODEL_SORT (
 		gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (contact_filter)));
 
@@ -1100,6 +1114,36 @@ setup_name_selector_model (ENameSelectorDialog *name_selector_dialog)
 	contact_selection_changed (name_selector_dialog);
 }
 
+static void
+shutdown_name_selector_model (ENameSelectorDialog *name_selector_dialog)
+{
+	gint i;
+
+	/* Rid UI of previous destination sections */
+
+	for (i = 0; i < name_selector_dialog->sections->len; i++)
+		free_section (name_selector_dialog, i);
+
+	g_array_set_size (name_selector_dialog->sections, 0);
+
+	/* Free sorting model */
+
+	if (name_selector_dialog->contact_sort) {
+		g_object_unref (name_selector_dialog->contact_sort);
+		name_selector_dialog->contact_sort = NULL;
+	}
+
+	/* Free backend model */
+
+	if (name_selector_dialog->name_selector_model) {
+		g_signal_handlers_disconnect_matched (name_selector_dialog->name_selector_model,
+						      G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, name_selector_dialog);
+
+		g_object_unref (name_selector_dialog->name_selector_model);
+		name_selector_dialog->name_selector_model = NULL;
+	}
+}
+
 static void
 deep_free_list (GList *list)
 {
@@ -1217,15 +1261,33 @@ e_name_selector_dialog_set_model (ENameSelectorDialog *name_selector_dialog,
 	if (model == name_selector_dialog->name_selector_model)
 		return;
 
-	g_signal_handlers_disconnect_matched (name_selector_dialog->name_selector_model,
-					      G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, name_selector_dialog);
-
-	g_object_unref (name_selector_dialog->name_selector_model);
+	shutdown_name_selector_model (name_selector_dialog);
 	name_selector_dialog->name_selector_model = g_object_ref (model);
 
 	setup_name_selector_model (name_selector_dialog);
 }
 
+/**
+ * e_name_selector_dialog_set_destination_index:
+ * @name_selector_dialog: an #ENameSelectorDialog
+ * @index: index of the destination section, starting from 0.
+ *
+ * Sets the index number of the destination section. 
+ **/
+void
+e_name_selector_dialog_set_destination_index (ENameSelectorDialog *name_selector_dialog,
+					      guint                index)
+{
+	ENameSelectorDialogPrivate *priv;
+
+	priv = E_NAME_SELECTOR_DIALOG_GET_PRIVATE (name_selector_dialog);
+
+	if (index >= name_selector_dialog->sections->len)
+		return;
+
+	priv->destination_index = index;
+}
+
 /* ----------------------------------- *
  * Widget creation functions for Glade *
  * ----------------------------------- */
