@@ -28,9 +28,17 @@
 #include <time.h>
 #include <unistd.h>
 #include <sys/types.h>
+#include <glib.h>
+
+#ifndef G_OS_WIN32
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#else
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#include <windns.h>
+#endif
 
 #include "e2k-context.h"
 #include "e2k-encoding-utils.h"
@@ -51,6 +59,25 @@
 #include <libxml/tree.h>
 #include <libxml/xmlmemory.h>
 
+#ifdef G_OS_WIN32
+/* The strtok() in Microsoft's C library is MT-safe (not stateless,
+ * but that is not needed here).
+ */
+#define strtok_r(s,sep,lasts ) (*(lasts) = strtok((s),(sep)))
+#endif
+
+#ifdef G_OS_WIN32
+#define CLOSE_SOCKET(socket) closesocket (socket)
+#define STATUS_IS_SOCKET_ERROR(status) ((status) == SOCKET_ERROR)
+#define SOCKET_IS_INVALID(socket) ((socket) == INVALID_SOCKET)
+#define BIND_STATUS_IS_ADDRINUSE() (WSAGetLastError () == WSAEADDRINUSE)
+#else
+#define CLOSE_SOCKET(socket) close (socket)
+#define STATUS_IS_SOCKET_ERROR(status) ((status) == -1)
+#define SOCKET_IS_INVALID(socket) ((socket) < 0)
+#define BIND_STATUS_IS_ADDRINUSE() (errno == EADDRINUSE)
+#endif
+
 #define PARENT_TYPE G_TYPE_OBJECT
 static GObjectClass *parent_class;
 
@@ -83,6 +110,9 @@ struct _E2kContextPrivate {
 #define E2K_CONTEXT_MIN_BATCH_SIZE 25
 #define E2K_CONTEXT_MAX_BATCH_SIZE 100
 
+/* For soup sync session timeout */
+#define E2K_SOUP_SESSION_TIMEOUT 30
+
 #ifdef E2K_DEBUG
 char *e2k_debug;
 int e2k_debug_level;
@@ -222,7 +252,7 @@ got_connection (SoupSocket *sock, guint status, gpointer user_data)
 	local_ipaddr = soup_address_get_physical (addr);
 
 	s = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
-	if (s == -1)
+	if (SOCKET_IS_INVALID (s))
 		goto done;
 
 	memset (&sin, 0, sizeof (sin));
@@ -235,14 +265,18 @@ got_connection (SoupSocket *sock, guint status, gpointer user_data)
 			port += 1024;
 		sin.sin_port = htons (port);
 		ret = bind (s, (struct sockaddr *)&sin, sizeof (sin));
-	} while (ret == -1 && errno == EADDRINUSE);
+	} while (STATUS_IS_SOCKET_ERROR (ret) && BIND_STATUS_IS_ADDRINUSE ());
 
 	if (ret == -1) {
-		close (s);
+		CLOSE_SOCKET (s);
 		goto done;
 	}
 
+#ifndef G_OS_WIN32
 	ctx->priv->listener_channel = g_io_channel_unix_new (s);
+#else
+	ctx->priv->listener_channel = g_io_channel_win32_new_socket (s);
+#endif
 	g_io_channel_set_encoding (ctx->priv->listener_channel, NULL, NULL);
 	g_io_channel_set_buffered (ctx->priv->listener_channel, FALSE);
 
@@ -326,6 +360,7 @@ e2k_context_set_auth (E2kContext *ctx, const char *username,
 		      const char *domain, const char *authmech,
 		      const char *password)
 {
+	guint timeout = E2K_SOUP_SESSION_TIMEOUT;
 
 	g_return_if_fail (E2K_IS_CONTEXT (ctx));
 
@@ -350,8 +385,15 @@ e2k_context_set_auth (E2kContext *ctx, const char *username,
 	if (ctx->priv->async_session)
 		g_object_unref (ctx->priv->async_session);
 
+	/* Set a default timeout value of 30 seconds.
+	   FIXME: Make timeout configurable 
+	*/
+	if (g_getenv ("SOUP_SESSION_TIMEOUT"))
+		timeout = atoi (g_getenv ("SOUP_SESSION_TIMEOUT"));
+	
 	ctx->priv->session = soup_session_sync_new_with_options (
 		SOUP_SESSION_USE_NTLM, !authmech || !strcmp (authmech, "NTLM"),
+		SOUP_SESSION_TIMEOUT, timeout,
 		NULL);
 	g_signal_connect (ctx->priv->session, "authenticate",
 			  G_CALLBACK (session_authenticate), ctx);
@@ -501,7 +543,7 @@ e2k_context_fba (E2kContext *ctx, SoupMessage *failed_msg)
 {
 	static gboolean in_fba_auth = FALSE;
 	int status, len;
-	char *body;
+	char *body = NULL;
 	char *action, *method, *name, *value;
 	xmlDoc *doc = NULL;
 	xmlNode *node;
